/* Copyright 2004 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: App.cpp,v 1.3 2004/10/17 21:42:04 tonic Exp $
 * $Revision: 1.3 $
 */

#include <time.h>
#include "main.h"


#define BLURMIX(a,b,n) ((a) * ((n) - 1) + (b)) / (n)


App * App::mSingleton = NULL;


App::App() :
    mState(0),
    mScreen(NULL),
    mAlive(TRUE),
    mTime(0),
    mStartTime(0),
    mNextUpdateTime(0),
    mCurrentStateHandler(NULL),
    mCustomEventFilter(NULL)
{
}


App::~App()
{
}


void App::setInstance(App *singleton)
{
    assert(mSingleton == NULL);
    mSingleton = singleton;
}


static int sEventFilter(const SDL_Event * /*event*/)
{
    return 1;
}


BOOL App::init()
{
    setInstance(this);

    SDL_WM_SetCaption(getCaption(), "");

    mScreen = SDL_GetVideoSurface();
    assert(mScreen != NULL);

    SDL_SetEventFilter(sEventFilter);

    resetTime();
    srand(time(NULL));

    return TRUE;
}


void App::run()
{
    while (isAlive())
    {
        Uint32 frameUpdateStart = SDL_GetTicks();
        SDL_Event event;
        while (SDL_PollEvent(&event))
        {
            SDLKey keysym;
            switch (event.type)
            {
            case SDL_KEYDOWN:
                keysym = event.key.keysym.sym;
                if (keysym == SDLK_ESCAPE)
                    setAlive(FALSE);
                if (mCurrentStateHandler != NULL)
                    mCurrentStateHandler->onKeyDown(keysym);
                break;

            case SDL_KEYUP:
                keysym = event.key.keysym.sym;
                if (mCurrentStateHandler != NULL)
                    mCurrentStateHandler->onKeyUp(keysym);
                break;

            case SDL_MOUSEMOTION:
                if (mCurrentStateHandler != NULL)
                    mCurrentStateHandler->onMouseMotion(event.motion);
                break;

            case SDL_MOUSEBUTTONUP:
                if (mCurrentStateHandler != NULL)
                    mCurrentStateHandler->onMouseUp(event.button);
                break;
                
            case SDL_MOUSEBUTTONDOWN:
                if (mCurrentStateHandler != NULL)
                    mCurrentStateHandler->onMouseDown(event.button);
                break;

            case SDL_QUIT:
                setAlive(FALSE);
                break;
            }
        }

        BOOL updated = FALSE;
        UINT32 time = SDL_GetTicks() - mStartTime;
        mTime = BLURMIX(mTime, time, 4);

        updated |= update();

        if (updated)
        {
            if (mCurrentStateHandler != NULL)
                mCurrentStateHandler->render(mScreen);
            //SDL_Flip(mScreen);    // SDL_UpdateRect is called from GameState
            glFlush();
            SDL_GL_SwapBuffers();
        }
        //else
        //    SDL_Delay(1);

        Uint32 frameUpdateEnd = SDL_GetTicks();
        if (frameUpdateStart == frameUpdateEnd)
            SDL_Delay(1);

        GLenum error = glGetError();
        if (error != GL_NO_ERROR)
            errorMessage("GL Error", "GL Error");
    }
}


BOOL App::update()
{
    if (mTime < mNextUpdateTime)
        return FALSE;

    if (mCurrentStateHandler == NULL)
    {
        mNextUpdateTime = mTime;
        return FALSE;
    }

    // TODO: Update this so that FALSE is returned if the current
    //       state handler returned FALSE for each iteration?
    while (mNextUpdateTime < mTime)
    {
        mNextUpdateTime += 1000 / mCurrentStateHandler->getUpdateFPS();
        
        mCurrentStateHandler->update(mNextUpdateTime);
    }

    return TRUE;
}


void App::resetTime()
{
    mNextUpdateTime = 0;
    mStartTime = SDL_GetTicks();
    mTime = 0;
}


void App::setState(STATE_TYPE state)
{
    if (mCurrentStateHandler != NULL)
        mCurrentStateHandler->onDeactivate();

    mCurrentStateHandler = getStateHandler(state);

    mState = state;
    if (mCurrentStateHandler != NULL)
        mCurrentStateHandler->onActivate(mTime);
}
